/*****************************************************************************
   KXTJ3.h

   Copyright (c) 2021 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
 ******************************************************************************/
#ifndef KXTJ3_H__
#define KXTJ3_H__

#ifndef float32
#define float32                   float
#endif

#define KXTJ3_COMM_OK              (0)
#define KXTJ3_COMM_ERROR           (-1)
#define KXTJ3_WAI_ERROR            (-2)
#define KXTJ3_PARAM_ERROR          (-3)

#define KXTJ3_DEVICE_ADDRESS_0F    (0x0F)
#define KXTJ3_DEVICE_ADDRESS_0E    (0x0E)

#define KXTJ3_ADDR_HIGH            (true)
#define KXTJ3_ADDR_LOW             (false)

#define KXTJ3_XOUT_L               (0x06)
#define KXTJ3_WHO_AM_I             (0x0F)
#define KXTJ3_CNTL1                (0x1B)
#define KXTJ3_DATA_CNTL            (0x21)

#define KXTJ3_CNTL1_WUFE           (1 << 1)
#define KXTJ3_CNTL1_TDTE           (1 << 2)
#define KXTJ3_CNTL1_GSEL_2G        (0 << 2)
#define KXTJ3_CNTL1_GSEL_4G        (2 << 2)
#define KXTJ3_CNTL1_GSEL_8G        (4 << 2)
#define KXTJ3_CNTL1_GSEL_16G       (1 << 2)
#define KXTJ3_CNTL1_DRDYE          (1 << 5)
#define KXTJ3_CNTL1_RES_LOWPOWER   (0 << 6)
#define KXTJ3_CNTL1_RES_HIGHRESO   (1 << 6)
#define KXTJ3_CNTL1_PC1            (1 << 7)
#define KXTJ3_DATA_CNTL_OSA_50HZ   (2 << 0)

#define KXTJ3_WAI_VAL              (0x35)
#define KXTJ3_CNTL1_VAL            (KXTJ3_CNTL1_RES_LOWPOWER | KXTJ3_CNTL1_GSEL_2G)
#define KXTJ3_DATA_CNTL_VAL        (KXTJ3_DATA_CNTL_OSA_50HZ)

#define KXTJ3_CNTL1_EN16GMASK      (0x04)
#define KXTJ3_CNTL1_GSELMASK       (0x18)
#define KXTJ3_CNTL1_GSEL_14BIT     (0x18)
#define KXTJ3_CNTL1_RES_MASK       (0x40)
#define KXTJ3_DATA_CNTL_ODRCHECK   (4)
#define KXTJ3_DATA_CNTL_OSAA_CLEAR (0x07)

#define KXTJ3_DRIVER_VERSION       (1.1F)

#define KXTJ3_GSENS_2G             (2)
#define KXTJ3_GSENS_4G             (4)
#define KXTJ3_GSENS_8G             (8)
#define KXTJ3_GSENS_16G            (16)
#define KXTJ3_GSENS_ERROR          (0xFF)

#define KXTJ3_READ_DATA_SIZE       (6)
#define KXTJ3_AXIS_SIZE            (KXTJ3_READ_DATA_SIZE / 2)
#define KXTJ3_DIVIDE_SHIFT         (15)

class KXTJ3
{
    public:
        KXTJ3(bool addr_pin);
        ~KXTJ3();
        float32 get_version(void);
        int32_t init(void);
        int32_t start(void);
        int32_t stop(void);
        int32_t get_val(uint8_t *raw);
        int32_t get_val(float32 *acc);

    private:
        uint8_t  Address;
        uint32_t RegCtlVal;
        int32_t  GDivide; //Coeffcient for converting from LSB into G

        int32_t init_setting_ic(uint8_t *checkval);
        int32_t init_setting_gval(uint8_t checkval);
        int32_t write(uint8_t reg, uint8_t *data, int32_t size);
        int32_t read(uint8_t reg, uint8_t *data, int32_t size);
};

#endif // KXTJ3_H__