/*****************************************************************************
   KXTJ3.ino

   Copyright (c) 2021 ROHM Co.,Ltd.

   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to deal
   in the Software without restriction, including without limitation the rights
   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
   copies of the Software, and to permit persons to whom the Software is
   furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
   THE SOFTWARE.
 ******************************************************************************/
#include <Arduino.h>
#include <HardwareSerial.h>
#include <Wire.h>
#include <KXTJ3.h>

#define SYSTEM_BAUDRATE (115200)
#define SYSTEM_WAIT     (100)    // ms
#define ERROR_WAIT      (1000)   // ms

KXTJ3 kxtj3(KXTJ3_ADDR_LOW);
void error_func(int32_t result);

void setup()
{
    int32_t result;
    float32 version;

    Serial.begin(SYSTEM_BAUDRATE);
    while (!Serial) {
    }

    (void)Serial.println("KXTJ3-1057 Sample Code Version : 1.1");

    version = kxtj3.get_version();
    (void)Serial.print("KXTJ3-1057 Driver Version : ");
    (void)Serial.println(version, 1);

    Wire.begin();

    result = kxtj3.init();
    if (result == KXTJ3_COMM_OK) {
        (void)kxtj3.start();
    } else {
        error_func(result);
    }
    
    return;
}

void loop()
{
    int32_t result;
    float32   acc[KXTJ3_AXIS_SIZE];

    result = kxtj3.get_val(acc);
    if (result == KXTJ3_COMM_OK) {
        (void)Serial.print("X(g):Y(g):Z(g) = ");
        (void)Serial.print(acc[0]);
        (void)Serial.print(", ");
        (void)Serial.print(acc[1]);
        (void)Serial.print(", ");
        (void)Serial.print(acc[2]);
        (void)Serial.println();
    } else {
        (void)Serial.println("Error : Can't get sensor data!");
        error_func(result);
    }

    delay(SYSTEM_WAIT);

    return;
}

void error_func(int32_t result)
{
    uint8_t cnt;

    switch (result) {
        case KXTJ3_COMM_ERROR:
            (void)Serial.println("Communication Error.");
            break;

        case KXTJ3_WAI_ERROR:
            (void)Serial.println("ID Error.");
            break;

        case KXTJ3_PARAM_ERROR:
            (void)Serial.println("Parameter Error.");
            break;

        default:
            (void)Serial.println("Unknown Error.");
            break;
    }

    (void)Serial.println("KXTJ3-1057 Check System and Driver Parameter.");
    cnt = 0;
    while (1) {
        (void)Serial.print(".");
        if (cnt < 30) {
            cnt++;
        } else {
            cnt = 0;
            (void)Serial.println();
        }
        delay(ERROR_WAIT);
    }

    return;
}

